#!/usr/bin/perl
# Author: Mariusz Strzelecki <szczeles@mat.umk.pl>
# Don't use Nagios embedded perl
# nagios: -epn

use strict;
our $plugin_version = '2.0';

# Let $Bin store absolute path of script directory
use FindBin '$Bin';

# Look for commons.pm in following directories
use lib "$Bin";
use lib "$Bin/..";

# Load common functions
use commons qw(%config &exit_plugin &setup_plugin &message &check_conditions &create_temp_file &check_config &run &check_file_existence);

# Load configuration and check for commandline arguments
setup_plugin("$Bin/check_sms.README", $plugin_version);
check_config('SMS_ADDRESS');
check_config('FILE_SIZE_KBS');

# Execute something
our $TMPFILE = create_temp_file("check_sms$$");
our $DATAFILE = create_temp_file("check_sms$$.file");
our $DATAFILE2 = create_temp_file("check_sms$$.file.downloaded");
our $REMOTEFILE = "check_sms$$";

# Generate random file
print `dd if=/dev/urandom of=$main::DATAFILE bs=1024 count=$config{'FILE_SIZE_KBS'} >/dev/null 2>&1`;
if ($? != 0) {
  exit_plugin("CRITICAL: Unable to generate random file to test!");
}

# Upload file
run("ucc", "put-file -s $main::DATAFILE -t $config{'SMS_ADDRESS'}#$main::REMOTEFILE -v -y", $TMPFILE, 1);

our @conditions = (
  {
    'test'       => '`grep -c "SMS contacted, server reply" $main::TMPFILE` != 1',
    'output'     => "CRITICAL: Unable to get access to the SMS at $config{'SMS_ADDRESS'}",
    'show_debug' => '`echo "See log from ucc for details:"; cat $main::TMPFILE`'
  },
  {
    'test'       => '`grep -c "java.io.IOException: Can\'t write data, server returned Internal Server Error" $main::TMPFILE` != 0',
    'output'     => "CRITICAL: Unable to write file at server side on $config{'SMS_ADDRESS'}",
    'show_debug' => '`echo "See log from ucc for details:"; cat $main::TMPFILE`'
  },
  {
    'test'       => '`grep -c "Can\'t put file." $main::TMPFILE` != 0',
    'output'     => "CRITICAL: Unable to upload file to $config{'SMS_ADDRESS'}, reason uknown",
    'show_debug' => '`echo "See ucc log to determine error:"; cat $main::TMPFILE`'
  },
  {
    'test'       => '`grep -c "is not a valid location." $main::TMPFILE` != 0',
    'output'     => "CRITICAL: Unable to upload file - $config{'SMS_ADDRESS'}/$main::REMOTEFILE is not a valid location.",
    'show_debug' => '`echo -e "Probably SMS is not available, see ucc log:"; cat $main::TMPFILE`'
  }
);
check_conditions(\@main::conditions);
my $UPLOADRATE=`tail -3 $main::TMPFILE | head -1`;
$UPLOADRATE =~ s/^Rate: (.*) kB\/sec\.\s+$/\1/;
$UPLOADRATE =~ s/[^0-9\.]//g;

our $CLEANUP='run("ucc", "rm -q $config{\'SMS_ADDRESS\'}#$main::REMOTEFILE");';

# Download file
run("ucc", "get-file -t $main::DATAFILE2 -s $config{'SMS_ADDRESS'}#$main::REMOTEFILE -v -y", $TMPFILE, 1);

our @conditions = (
  {
    'test'       => '`file -b $main::DATAFILE` =~ "empty"',
    'output'     => "CRITICAL: Download succeeded, but downloaded file is empty ($main::REMOTEFILE on $config{'SMS_ADDRESS'})",
    'show_debug' => 0
  }
);
check_conditions(\@main::conditions);
my $DOWNLOADRATE=`tail -2 $main::TMPFILE | head -1`;
$DOWNLOADRATE =~ s/^Rate: (.*) kB\/sec\.\s+$/\1/;
$DOWNLOADRATE =~ s/[^0-9\.]//g;

# Compare files
`cmp -s $main::DATAFILE $main::DATAFILE2`;

if ( $? != 0 ) {
  $CLEANUP = '';
  our @conditions = (
    {
      'test'       => '$? != 0',
      'output'     => "CRITICAL: Downloaded file differs from uploaded one.",
      'show_debug' => "`echo Check files $main::DATAFILE, $main::DATAFILE2 on local machine and $main::REMOTEFILE on $config{'SMS_ADDRESS'}. Files aren't deleted!`"
    }
  );
  check_conditions(\@main::conditions);
}

# If executions goes here, it's great
exit_plugin("OK: SMS at $config{'SMS_ADDRESS'} works with U:$UPLOADRATE kB/s, D:$DOWNLOADRATE kB/s | UploadRate=${UPLOADRATE}KB DownloadRate=${DOWNLOADRATE}KB");
