#!/usr/bin/perl
# Author: Mariusz Strzelecki <szczeles@mat.umk.pl>
# Don't use Nagios embedded perl
# nagios: -epn

use strict;
our $plugin_version = '2.0';

# Let $Bin store absolute path of script directory
use FindBin '$Bin';

# Look for commons.pm in following directories
use lib "$Bin";
use lib "$Bin/..";

# Load common functions
use commons qw(%config &exit_plugin &setup_plugin &message &check_conditions &create_temp_file &check_config &run &check_file_existence &is_debug_enabled);

# Load configuration and check for commandline arguments
setup_plugin("$Bin/check_workflow.README", $plugin_version);

check_config("WORKFLOWSERVICE_NAME, GLOBAL_STORAGE_URL, TEST_FILE_PATH, CACHE_FILE");

my $script_location = "$Bin/check_workflow_test.groovy";
check_file_existence($script_location);
my $test_file = $config{'TEST_FILE_PATH'};
check_file_existence($test_file);
`touch $config{'CACHE_FILE'}`;

# Execute something
our $TMPFILE = create_temp_file("check_workflow$$");
my $start = time();
run("ucc", "run-groovy -f $script_location $config{'WORKFLOWSERVICE_NAME'} $test_file $config{'GLOBAL_STORAGE_URL'} ".($config{'originalTimeout'}-100)." $config{'CACHE_FILE'} ".(is_debug_enabled()?1:0), $TMPFILE);
my $uccTime = time() - $start;
our $result = `head -1 $main::TMPFILE`;
chomp $result;

# Check conditions
our @conditions = (
  {
    'test'       => '$main::result =~ /Wokflow succeeded! Time elapsed:/ || $main::result =~ /Workflow submission succeded/',
    'output'     => "OK: $main::result",
    'show_debug' => '`tail -n +2 $main::TMPFILE`'
  },
  {
    'test'       => '$main::result =~ /Can\'t create list of target systems./ || $main::result =~ /^Could not invoke service.. Nested exception is org.codehaus.xfire.fault.XFireFault: Couldn\'t send message./',
    'output'     => "CRITICAL: Unable to connect to the Registry!",
    'show_debug' => '`cat $main::TMPFILE`'
  },
  {
    'test'       => '$main::result =~ /Some TSFs are not available in the Registry/ || $main::result =~ /Unable to list target systems from/ || $main::result =~ /Unable to create TSS in/ || $main::result =~ /There are TSFs registered in the Registry, but unable to contact/',
    'output'     => "CRITICAL: $main::result!",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /Could not perform truncAppend/ || $main::result =~ /Can\'t get protocols/',
    'output'     => "CRITICAL: Unable to upload test file: $main::result",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /WorkflowFactory service exists in Registry but doesn\'t respond/',
    'output'     => "CRITICAL: WorkflowFactory does not respond!",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /Workflow is still running after/ || $main::result =~ /Unable to upload file/ || $main::result =~ /Workflow at .* does not exists/',
    'output'     => "CRITICAL: $main::result!",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /Workflow execution failed/',
    'output'     => "CRITICAL: Workflow execution does not seem successful!",
    'show_debug' => '`cat $main::TMPFILE`'
  },
  {
    'test'       => '$main::result =~ /Could not resolve location, result:/',
    'output'     => "CRITICAL: Could not resolve output file location using LocationManager!",
    'show_debug' => '`cat $main::TMPFILE`'
  },
  {
    'test'       => '$main::result =~ /Tracer service is not/',
    'output'     => "CRITICAL: $main::result",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /Unexpected output/',
    'output'     => "CRITICAL: $main::result!",
    'show_debug' => 0
  },
  {
    'test'       => '$main::result =~ /Workflow succeded, but some TSFs available in the Registry were not used/',
    'output'     => "WARNING: $main::result",
    'show_debug' => 0
  },
);
check_conditions(\@main::conditions);

# If executions goes here, it is abnormal situation...
exit_plugin("UNKNOWN: Unknown error", '`echo "See log for details:"; cat $main::TMPFILE`');
