import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.chemomentum.common.ws.IWorkflowFactory;
import org.chemomentum.common.impl.workflow.WorkflowSubmissionClient;
import org.chemomentum.common.impl.workflow.WorkflowManagementClient;
import org.chemomentum.common.api.workflow.WorkflowWrapper;
import de.fzj.unicore.ucc.helpers.TargetSystemFactoryLister;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.ucc.util.Mode;
import de.fzj.unicore.ucc.util.ImportFromLocal;
import org.chemomentum.workflow.xmlbeans.WorkflowDocument;
import org.apache.xmlbeans.XmlObject;
import org.chemomentum.workflow.xmlbeans.RulesDocument;
import org.chemomentum.workflow.xmlbeans.StatusType;
import de.fzj.unicore.uas.TargetSystemFactory;
import org.chemomentum.common.ws.ILocationManager;
import org.chemomentum.dataManagement.locationManager.QueryLocationRequestDocument;
import org.chemomentum.workflow.xmlbeans.WorkflowResourcePropertiesDocument;
import org.chemomentum.tracer.xmlbeans.FilterEntriesRequestDocument;
import org.chemomentum.common.ws.ITrace;
import org.chemomentum.tracer.xmlbeans.EntryDocument.Entry;
import org.unigrids.services.atomic.types.ProtocolType;
import eu.unicore.security.xfireutil.client.SecuredXFireClientFactory;
import groovy.xml.QName;
import javax.xml.bind.DatatypeConverter;
import java.util.Calendar;

class WorkflowTester {
  // global UCC params
  private registry;
  private securityProperties;
  private messageWriter;
  private verbosity;

  // sms client and workflow file name are common for most operations
  private smsClient = null;
  private workflowFileName = null;
  public static final String textToWrite = "UNICORE Workflow Test";

  // output statistics
  private workflowDefaultTT = null;
  private additional = null;
  private performanceData = '';
  private overallStatus = '';
  private submittedWorkflowID = '';

  // placeholders for excetepions messages
  private checkStatus = '';
  private submitStatus = '';

  public WorkflowTester(registry, securityProperties, messageWriter, verbosity, workflowFileName, storageUrl) {
    this.registry = registry;
    this.securityProperties = securityProperties;
    this.messageWriter = messageWriter;
    this.verbosity = verbosity;
    this.workflowFileName = workflowFileName;

    def smsEpr = EndpointReferenceType.Factory.newInstance();
    smsEpr.addNewAddress().setStringValue(storageUrl);
    this.smsClient = new StorageClient(storageUrl, smsEpr, securityProperties);

    additional = this.makeConnectAndSystemInfoCheck(workflowFileName);
  }
  
  public finalizeTests() {
    if (checkStatus == '' && submitStatus == '') { // no exception at all
      if (overallStatus == '') {
        messageWriter.message("Workflow submission succeded!");
      } else {
        messageWriter.message(overallStatus+performanceData);
      }
      messageWriter.message("Default Termination Time: "+workflowDefaultTT+" day"+(workflowDefaultTT > 1 ? "s" : "")+".");
    } else {
      throw new Exception(checkStatus+submitStatus);
    }
  }

  public clearData(wmClient) {
    if (wmClient != null && wmClient.checkConnection() == true) {
      try {
        smsClient.delete(this.getInputFileName(wmClient.getWorkflowID()));
      } catch (Exception e) { }

      try {
        smsClient.delete(wmClient.getWorkflowID());
      } catch (Exception e) { }

      messageWriter.verbose("Files deleted");

      wmClient.destroy();
      messageWriter.verbose("Workflow destroyed");
    }
  }

  public submitWorkflow(workflowServiceUrl) {
    def id = '';
    try {
      id = this.createWorkflow(this.createWSC(workflowServiceUrl), workflowFileName);
    } catch (Exception e) {
      this.submitStatus = e.getMessage();
    } finally {
      return id;
    }
  }

  public checkStatusOfWorkflow(workflowUrl) {
    def wmClient;
    try {
      wmClient = this.createWMC(workflowUrl)

      try {
        def status = wmClient.status;
        if (status == StatusType.RUNNING) {
          throw new Exception("Workflow is still running after "+(-(wmClient.getResourcePropertiesDocument().getWorkflowResourceProperties().getSubmissionTime().getTimeInMillis() -  Calendar.getInstance().getTimeInMillis())/1000) + " seconds");
        } else if (status != StatusType.SUCCESSFUL) {
          throw new Exception("Workflow execution failed! Error message is:\n" + wmClient.detailedStatus.toString());
        }
      } catch (de.fzj.unicore.wsrflite.xmlbeans.exceptions.ResourceUnknownFault e) {
        throw new Exception('Workflow at '+workflowUrl+' does not exists');
      }
      messageWriter.verbose "Workflow succeded!"

      this.downloadAndCheckFile(wmClient.getWorkflowID());
      this.traceWorkflow(wmClient.getTracerAddress(), wmClient.getWorkflowID());
    } catch (Exception e) {
      this.checkStatus = e.getMessage();
    } finally {
      this.clearData(wmClient);
    }
  }

  private createWSC(workflowServiceUrl) {
    def wsEpr = EndpointReferenceType.Factory.newInstance();
    wsEpr.addNewAddress().setStringValue(workflowServiceUrl);
    return new WorkflowSubmissionClient(workflowServiceUrl, wsEpr, securityProperties);
  }

  private createWMC(workflowUrl) {
    def workflowEpr = EndpointReferenceType.Factory.newInstance();
    workflowEpr.addNewAddress().setStringValue(workflowUrl);
    return new WorkflowManagementClient(workflowEpr.getAddress().getStringValue(), workflowEpr, securityProperties);
  }

  private downloadAndCheckFile(workflowID) {
    /************ querying location mapper****************/

    def available=registry.listServices(ILocationManager.PORT);

    if (available.size() == 0) {
      throw new Exception("Location mapping service is not available!");
    }
    def lm = new SecuredXFireClientFactory(securityProperties).createPlainWSProxy(ILocationManager.class, available.get(0).getAddress().getStringValue());
    def req=QueryLocationRequestDocument.Factory.newInstance();
    req.addNewQueryLocationRequest().addNewInput().setLogicalName('c9m:'+workflowID+'/fileX.out');
    def res=lm.queryLocation(req);

    if (res.getQueryLocationResponse().getFaultsOccurred()){
      throw new Exception("Could not resolve location, result "+res);
    }

    def String[] location=res.getQueryLocationResponse().getResultArray(0).getPhysicalLocationArray()[0].split('#');
    messageWriter.verbose 'Location mapped as '+location[1];

    /************ downlaoding output file and comparing ************/

    def ftc = smsClient.getExport(location[1], [ProtocolType.BFT,ProtocolType.RBYTEIO] as ProtocolType.Enum[]);
    def fos = new ByteArrayOutputStream();
    ftc.readAllData(fos);
    messageWriter.verbose "Download succeded"

    def result  = fos.toString();
    if (!result.contains(textToWrite)) {
      verbosity = 1;
      throw new Exception('Unexpected output: sent "' + textToWrite + '" but received "' + result +'"');
    }
    messageWriter.verbose "File content matches!"
  }

  private traceWorkflow(tracerEPR, workflowID) {
    /************ getting statistical data - workflow trace *********/
    if (tracerEPR == null) {
      throw new Exception('Tracer service is not available!');
    }
    def tracer=new SecuredXFireClientFactory(securityProperties).createPlainWSProxy(ITrace.class, tracerEPR.getAddress().getStringValue());

    def ferd = FilterEntriesRequestDocument.Factory.newInstance();
    def fer = ferd.addNewFilterEntriesRequest();
    fer.setField(ITrace.MESSAGE_ID);
    fer.setPattern("%"+workflowID+"%");
    def start = null, end = null;
    def extraData = "";
    def realTime = 0.0;
    try {
      Entry[] entries=tracer.filterEntries(ferd).getFilterEntriesResponse().getTrace().getEntryArray();
      def lastSite = "";
      def startTime = null;
      entries.each {
        if (it.getOperation().equalsIgnoreCase("submitWorkflow"))
          start = it.getTimeStamp();
        else {
          end = it.getTimeStamp();

          if (it.getOperation().equalsIgnoreCase("Submit")) {
            lastSite = it.getTargetID().toString();
            startTime = it.getTimeStamp();
          } else if (it.getOperation().equalsIgnoreCase("finishedJob") && lastSite.equalsIgnoreCase(it.getSourceID())) {
            def tokens = lastSite.split("/");
            extraData += " "+tokens[3] + "=" + ((it.getTimeStamp().getTimeInMillis() - startTime.getTimeInMillis()) / 1000) + "s";
            realTime += (it.getTimeStamp().getTimeInMillis() - startTime.getTimeInMillis()) / 1000;
          }
        }
      }
    } catch (Exception e) {
      throw new Exception("Tracer service is not responding: "+e.getMessage());
    }

    /************* looking for previous warning and finishing *********/

    def time = (end.getTimeInMillis() - start.getTimeInMillis()) / 1000;
    this.performanceData = " | WorkflowExecutionTime="+time+"s WorkflowOverhead="+(time-realTime)+"s"+extraData;
    if (additional.size() != 0) {
      throw new Exception('Workflow succeded, but some TSFs available in the Registry were not used: '+additional + ". Time elapsed: " +time+" sec."+this.performanceData);
    }

    this.overallStatus = "Wokflow succeeded! Time elapsed: " +time+" sec.";
  }

  private createWorkflow(wsClient, workflowFileName) {
    /************ preparing to workflow submission************/
    def content = new File(workflowFileName).text;

    def String workflowName = "UMI 2.0 check_workflow test, submitted at "+(new Date().format('yyyy-MM-dd HH:mm:ss'));
    if (wsClient == null || wsClient.checkConnection() != true) 
      throw new Exception("WorkflowFactory service exists in Registry but doesn't respond");
    def wmClient = wsClient.createWorkflow(workflowName, null, smsClient.getEPR());

    def testFileName = this.uploadTestFile(wmClient.getWorkflowID());

    /**************** checking default TT *******************/

    def myTT = wmClient.getTerminationTime();
    myTT.add(Calendar.MINUTE, 10); // sumbission overhead
    workflowDefaultTT = ((myTT.getTimeInMillis() - Calendar.getInstance().getTimeInMillis())/86400000).intValue();

    def terminationTime=Calendar.getInstance();
    terminationTime.add(Calendar.HOUR, 3);
    wmClient.setTerminationTime(terminationTime);

    /*************** generating workflow document ***********/
 
    def wd = WorkflowDocument.Factory.newInstance();
    def x = XmlObject.Factory.parse(content.toString().replace('u6://SHARE', 'BFT:'+smsClient.getUrl()+'#').replace('${WORKFLOW_ID}', wmClient.getWorkflowID()).replace('file0.out', testFileName));
    wd.addNewWorkflow().set(x);
    wd.getWorkflow().setDialect(x.newDomNode().getFirstChild().getNamespaceURI());

    def rd = RulesDocument.Factory.newInstance();
    rd.addNewRules();

    /**************** submitting workflow ************/

    def wf = new WorkflowWrapper(wd.getWorkflow(), rd.getRules());
    wmClient.submitWorkflow(wf);
    messageWriter.verbose "Workflow submitted with id "+wmClient.getWorkflowID()
    this.submittedWorkflowID = wmClient.getWorkflowID();
    return wmClient.epr.address.stringValue;
  }

  private getInputFileName(workflowID) {
    return 'input_for_workflow_' + workflowID + '.txt';
  }

  private uploadTestFile(workflowID) {
    def randFileName = this.getInputFileName(workflowID);
    try {
      def fis = new ByteArrayInputStream(textToWrite.getBytes());
      def ftc = smsClient.getImport(randFileName, [ProtocolType.BFT,ProtocolType.RBYTEIO] as ProtocolType.Enum[]);
      ftc.writeAllData(fis);
      ftc.destroy();
      fis.close();
    } catch (Exception e) {
      throw new Exception("Unable to upload file: "+e);
    }
    messageWriter.verbose "Upload succeded";
    return randFileName;
  }


  /**
   * Reads workflow file and checks if all targed systems are available.
   * Also returns list of target system that are available but are not used by workflow
   */
  private makeConnectAndSystemInfoCheck(workflowFileName) {
    def xml = new XmlParser().parse(new File(workflowFileName));
    def ns1 = new groovy.xml.Namespace('http://www.chemomentum.org/workflow/simple'); 
    def ns2 = new groovy.xml.Namespace('http://schemas.ggf.org/jsdl/2005/11/jsdl'); 
    def tsfsNeeded = [];
    def tsfsFound = [];
    def tsfsAdditional = [];
    xml[ns1.Activity].each {
      tsfsNeeded.push(it[ns1.JSDL][ns2.JobDescription][ns2.Resources][ns2.CandidateHosts][ns2.HostName].get(0).value().get(0));
    }
    def numberOfUsedTSFs = tsfsNeeded.size();
  
    def lister=new TargetSystemFactoryLister(registry,securityProperties,messageWriter);
    if (registry.checkConnection() == false)
      throw new Exception('Can\'t create list of target systems.');

    lister.each {
      def url = it.epr.address.stringValue;
      if (tsfsNeeded.contains(url)) {
        def tsss = it.accessibleTargetSystems;
        if (tsss == null) {
          throw new Exception('Unable to list target systems from: '+url);
        }
        def num = tsss.size();
        if (num >= 1) 
          messageWriter.verbose "Found "+num+" TSS at "+url;
        else {
          messageWriter.verbose "None TSS found, creating one at "+url;
          try {
            it.createTSS();
          } catch (Exception e) {
            throw new Exception("Unable to create TSS in "+url+", reason: "+e.getMessage());
          }
        }
        tsfsNeeded -= url;
      }
      if (!tsfsFound.contains(url)) tsfsFound.push(url);
    }
    
    if (tsfsNeeded.size() != 0) {

      /**************** system-info ************/

      def eprs = registry.listServices(TargetSystemFactory.TSF_PORT);
      def unaccessible = [];

      eprs.each {
        def url = it.address.stringValue;
        if (tsfsNeeded.contains(url)) {
          tsfsNeeded -= url;
          unaccessible.push(url);
        }
      }
      if (tsfsNeeded.size() != 0) {
        throw new Exception('Some TSFs are not available in the Registry: '+tsfsNeeded);
      }

      if (unaccessible.size() != 0) {
        throw new Exception('There are TSFs registered in the Registry, but unable to contact with: '+unaccessible);
      }
    }

    if (numberOfUsedTSFs < tsfsFound.size()) {
      xml[ns1.Activity].each {
        tsfsFound -= it[ns1.JSDL][ns2.JobDescription][ns2.Resources][ns2.CandidateHosts][ns2.HostName].get(0).value().get(0);
      }
      tsfsAdditional = tsfsFound;
    }
    return tsfsAdditional;
  }
}




def Boolean verbosity = false;
def tester = null;

try {
  if (commandLine.getArgs().size() != 7)
    throw new Exception("Usage: run-groovy -f test_workflow.groovy WORKFLOW_SERVICE_NAME WORFKLOW_SWF_FILE STORAGE_NAME WORKFLOW_TIMEOUT CACHE_FILE VERBOSITY_FLAG");

  def String workflowServiceName = commandLine.args[1];
  def String workflowFileName = commandLine.args[2];
  def String smsUrl = commandLine.args[3];
  def String cacheFile = commandLine.args[5];
  verbosity = commandLine.args[6] == '1';

  file = new File(cacheFile);
  oldId = file.text;

  tester = new WorkflowTester(registry, securityProperties, messageWriter, verbosity, workflowFileName, smsUrl);
  
  if (oldId) 
    tester.checkStatusOfWorkflow(oldId);  
  file.setText(tester.submitWorkflow(workflowServiceName));
  tester.finalizeTests();

} catch (Exception e) {
  messageWriter.message e.message;
  return;
}
